from unittest import TestCase
from unittest.mock import patch

from events_protocol.client.event_client import EventClient
from events_protocol.client.exception.request_exception import BadProtocolException
from events_protocol.core.context import EventContext, EventContextHolder
from events_protocol.core.model.event import ResponseEvent
from events_protocol.core.model.event_type import EventErrorType


class EventClientTest(TestCase):
    def setUp(self):
        self.client = EventClient("http://test.com/events/")

    @patch("events_protocol.client.event_client.HttpClient.post")
    def test_success_response(self, post_method):
        event = {
            "name": "event:name",
            "version": 1,
            "payload": {"test": "test"},
        }
        response_event = ResponseEvent(
            name="event:name:response", version=1, payload={"test": "test"},
        )

        post_method.return_value = response_event.to_json()
        response = self.client.send_event(**event)

        self.assertTrue(response.is_success)
        self.assertEqual(response, response_event)

    @patch("events_protocol.client.event_client.HttpClient.post")
    def test_error_response(self, post_method):
        event = {
            "name": "event:name",
            "version": 1,
            "payload": {"test": "test"},
        }
        response_event = ResponseEvent(
            name="event:name:error", version=1, payload={"test": "test"},
        )

        post_method.return_value = response_event.to_json()
        response = self.client.send_event(**event)
        self.assertTrue(response.is_error)
        self.assertEqual(EventErrorType.GENERIC, response.error_type)
        self.assertEqual(response, response_event)

    @patch("events_protocol.client.event_client.HttpClient.post")
    def test_invalid_response(self, post_method):
        event = {
            "name": "event:name",
            "version": 1,
            "payload": {"test": "test"},
        }

        post_method.return_value = "{}"

        with self.assertRaises(BadProtocolException):
            self.client.send_event(**event)


class BuildEventTest(TestCase):
    def setUp(self):
        self.client = EventClient("http://test.com/events/")

    def test_build_event_with_context(self):
        context = EventContext(id="my-id", flow_id="my-flow-id", name="test")
        EventContextHolder.set(context)

        event = {
            "name": "event:name",
            "version": 1,
            "payload": {"test": "test"},
            "id": "id-test",
            "flow_id": "flow-id-test",
        }
        builded_event = self.client.build_request_event(**event)

        self.assertEqual(context.id, builded_event.id)
        self.assertEqual(context.flow_id, builded_event.flow_id)

        EventContextHolder.clean()

    def test_build_event_without_context(self):
        event = {
            "name": "event:name",
            "version": 1,
            "payload": {"test": "test"},
            "id": "test-id",
            "flow_id": "test-flow-id",
        }
        builded_event = self.client.build_request_event(**event)

        self.assertEqual(event["id"], builded_event.id)
        self.assertEqual(event["flow_id"], builded_event.flow_id)

    def test_build_event_without_pass_id_and_flow_id(self):
        event = {"name": "event:name", "version": 1, "payload": {"test": "test"}}
        builded_event = self.client.build_request_event(**event)

        self.assertIsNotNone(builded_event.id)
        self.assertIsNotNone(builded_event.flow_id)



from unittest import TestCase

from events_protocol.core.exception import EventNotFoundException
from events_protocol.core.logging.supressor import supress_log
from events_protocol.server.handler.event_handler import EventHandler
from events_protocol.server.handler.event_handler_discovery import EventDiscovery


class TestEventDiscovery(TestCase):
    @supress_log
    def test_add_event_handler_getting_correct_one(self):
        class FakeEventHandler(EventHandler):
            pass

        event_name = "put:test:here"
        event_version = 2
        expected_event_handler = FakeEventHandler
        EventDiscovery.add(event_name, expected_event_handler, version=event_version)
        event_handler = EventDiscovery.get(event_name, event_version)
        self.assertEqual(expected_event_handler, event_handler)

    @supress_log
    def test_add_event_handler_wrongly_raising_type_error(self):
        event_name = "put:test:here"
        event_version = 2
        event_handler = "A string is not a event, dude"

        with self.assertRaises(TypeError):
            EventDiscovery.add(event_name, event_handler, version=event_version)

    def test_add_event_handler_getting_raising_event_not_found_exception(self):
        class FakeEventHandler(EventHandler):
            pass

        event_name = "put:test:here"
        event_version = 2
        wrong_event_version = 3
        expected_event_handler = FakeEventHandler
        EventDiscovery.add(event_name, expected_event_handler, version=event_version)
        with self.assertRaises(EventNotFoundException):
            EventDiscovery.get(event_name, wrong_event_version)

    @supress_log
    def test_add_event_handler_with_wrong_name_raising_value_error(self):
        class FakeEventHandler(EventHandler):
            pass

        event_name = "wrong/mame:test:here"
        event_version = 2
        expected_event_handler = FakeEventHandler

        with self.assertRaises(ValueError):
            EventDiscovery.add(event_name, expected_event_handler, version=event_version)



import typing
from contextlib import asynccontextmanager, contextmanager
from contextvars import ContextVar
from uuid import UUID

from events_protocol.core.model.base import BaseModel

IdType = typing.Union[str, UUID]


class EventContext(BaseModel):
    id: typing.Optional[IdType]
    flow_id: typing.Optional[IdType]
    event_name: typing.Optional[str]
    event_version: typing.Optional[int]
    user_id: typing.Optional[str]


_context: ContextVar[EventContext] = ContextVar("event_context", default=None)


class EventContextHolder:
    @staticmethod
    def get() -> EventContext:
        return _context.get() or EventContext()

    @staticmethod
    def set(event_context: EventContext) -> None:
        _context.set(event_context)

    @staticmethod
    def clean() -> None:
        _context.set(None)

    @classmethod
    @contextmanager
    def with_context(
        cls,
        context_id: IdType,
        context_flow_id: IdType,
        event_name: str,
        event_version: int,
        user_id: str = None,
    ):
        try:
            event_context = EventContext(
                id=context_id,
                flow_id=context_flow_id,
                event_name=event_name,
                user_id=user_id,
                event_version=event_version,
            )
            cls.set(event_context)
            yield cls.get()
        finally:
            cls.clean()

    @classmethod
    @asynccontextmanager
    async def with_async_context(
        cls,
        context_id: IdType,
        context_flow_id: IdType,
        event_name: str,
        event_version: int,
        user_id: str = None,
    ):
        try:
            event_context = EventContext(
                id=context_id,
                flow_id=context_flow_id,
                event_name=event_name,
                user_id=user_id,
                event_version=event_version,
            )
            cls.set(event_context)
            yield cls.get()
        finally:
            cls.clean()



import json
import typing
from dataclasses import dataclass

from pydantic import BaseModel as PydanticBaseModel
from pydantic import validate_model


def _to_camel(string: str) -> str:
    result = string.split("_")
    result[1:] = [word.capitalize() for word in result[1:]]
    return "".join(result)


@dataclass
class Field:
    name: str
    error_type: str
    message: str

    def to_dict(self) -> typing.Dict[str, typing.Any]:
        return dict(name=self.name, errorType=self.error_type, message=self.message)


@dataclass
class ValidationError(Exception):
    fields: typing.List[Field]

    def to_dict(self) -> typing.Dict[str, typing.Any]:
        _fields = list()
        for item in self.fields:
            _fields.append(item.to_dict())
        return dict(fields=_fields)

    def to_json(self):
        return json.dumps(self.to_dict())


class BaseModel(PydanticBaseModel):
    def __init__(__pydantic_self__, **data):
        if typing.TYPE_CHECKING:
            __pydantic_self__.__dict__: typing.Dict[str, typing.Any] = {}
            __pydantic_self__.__fields_set__: "SetStr" = set()
        values, fields_set, validation_error = validate_model(__pydantic_self__.__class__, data)
        if validation_error:
            _fields = list()
            errors = validation_error.errors()
            for each_error in errors:
                _fields.append(
                    Field(
                        name=each_error.get("loc")[0],
                        message=each_error.get("msg"),
                        error_type=each_error.get("type"),
                    )
                )
            raise ValidationError(_fields)
        object.__setattr__(__pydantic_self__, "__dict__", values)
        object.__setattr__(__pydantic_self__, "__fields_set__", fields_set)

    @classmethod
    def from_object(cls, obj: "BaseModel"):
        return cls(**obj.to_dict())

    def to_dict(self, *args, **kwargs) -> typing.Dict[str, typing.Any]:
        return self.dict(by_alias=True, *args, **kwargs)

    def to_json(self, *args, **kwargs) -> str:
        return self.json(by_alias=True)


class CamelPydanticMixin(BaseModel):
    class Config:
        alias_generator = _to_camel

    def __init__(self, by_alias=True, **data: typing.Any) -> None:
        _data = dict()
        for key, value in data.items():
            if by_alias:
                key = self.Config.alias_generator(key)
            _data[key] = value
        super().__init__(**_data)

    @classmethod
    def from_json(cls, data: str) -> "CamelPydanticMixin":
        try:
            _data = json.loads(data)
        except json.decoder.JSONDecodeError as exception:
            raise ValidationError(
                fields=[Field(name="json", error_type=exception.msg, message=str(exception))]
            )
        return cls(**_data)



from typing import Dict, Optional
from uuid import uuid4

from ..core.context import EventContextHolder
from ..core.model.base import ValidationError
from ..core.model.event import RequestEvent, ResponseEvent
from .exception.request_exception import BadProtocolException
from .http import HttpClient


class EventClient:
    def __init__(self, url: str):
        self.url = url
        self.http_client = HttpClient()
        self.default_timeout = 60000

    def send_event(
        self,
        name: str,
        version: int,
        payload: Optional[Dict] = {},
        id: str = None,
        flow_id: str = None,
        identity: Dict = {},
        auth: Dict = {},
        metadata: Dict = {},
        timeout: Optional[int] = None,
    ) -> ResponseEvent:
        request_event = self.build_request_event(
            name, version, payload, id, flow_id, identity, auth, metadata
        )
        return self.send_request_event(request_event, timeout)

    def send_request_event(
        self, request_event: RequestEvent, timeout: Optional[int] = None
    ) -> ResponseEvent:
        response = self.http_client.post(
            url=self.url,
            headers={"Content-Type": "application/json", "charset": "UTF-8",},
            payload=request_event.to_json(),
            timeout=timeout or self.default_timeout,
        )
        return self.parse_event(response)

    def parse_event(self, raw_response: str) -> ResponseEvent:
        try:
            return ResponseEvent.from_json(raw_response)
        except ValidationError:
            raise BadProtocolException("Error on parsing event response")

    def build_request_event(
        self,
        name: str,
        version: int,
        payload: Dict,
        id: str = None,
        flow_id: str = None,
        identity: Dict = {},
        auth: Dict = {},
        metadata: Dict = {},
    ) -> RequestEvent:
        context = EventContextHolder.get()
        return RequestEvent(
            name=name,
            version=version,
            payload=payload,
            id=context.id or id or str(uuid4()),
            flow_id=context.flow_id or flow_id or str(uuid4()),
            identity=identity,
            auth=auth,
            metadata=metadata,
        )



__version__ = "0.1.6"  # pragma: no cover

from .core.builder import EventBuilder
from .client import EventClient
from .core.model.base import BaseModel
from .core.model.event import Event, ResponseEvent
from .server.handler.event_handler import EventHandler
from .server.handler.event_handler_registry import EventRegister
from .server.parser.event_processor import EventProcessor



from unittest import TestCase

from events_protocol.server.handler.event_handler import EventHandler
from events_protocol.server.handler.event_handler_discovery import EventDiscovery
from events_protocol.server.handler.event_handler_registry import EventRegister


class _TestEventHandler(EventHandler):
    async def handle():
        pass


class TestEventRegister(TestCase):
    def test_event_instantiation(self):
        class EventTester(EventRegister):
            event_name = "event:top:zera"
            event_version = 1
            event_handler = _TestEventHandler

        EventTester.register_event()
        event = EventDiscovery.get(EventTester.event_name, EventTester.event_version)
        self.assertEqual(event, _TestEventHandler)



import asyncio
from collections import ChainMap
from unittest import TestCase
from uuid import uuid4

from events_protocol.core.context import EventContext, EventContextHolder
from tests.utils.sync import make_sync


class TestEventContextHolder(TestCase):
    @make_sync
    async def test_with_context(self):
        test_event = EventContext(id="my_id", flow_id="my_flow_id", name="test")
        async with EventContextHolder.with_async_context(
            test_event.id, test_event.flow_id, test_event.event_name, test_event.event_version
        ) as event_context:
            self.assertEqual(event_context, test_event)
            self.assertEqual(EventContextHolder.get(), test_event)
        self.assertEqual(EventContextHolder.get(), EventContext())

    def test_multiple_async_context_not_interfering_with_each_other(self):
        _events = list()

        async def event_0():
            first_event = EventContext(id=uuid4(), flow_id=uuid4(), name="test")
            async with EventContextHolder.with_async_context(
                first_event.id,
                first_event.flow_id,
                first_event.event_name,
                first_event.event_version,
            ) as event_context:
                assert EventContextHolder.get() == event_context, "EventContext was wrongly set"
                _events.append({0: EventContextHolder.get()})
                await asyncio.sleep(0.2)

        async def event_1():
            second_event = EventContext(id=uuid4(), flow_id=uuid4(), name="test")
            async with EventContextHolder.with_async_context(
                second_event.id,
                second_event.flow_id,
                second_event.event_name,
                second_event.event_version,
            ) as event_context:
                assert EventContextHolder.get() == event_context, "EventContext was wrongly set"
                _events.append({1: EventContextHolder.get()})

        async def event_2():
            _events.append({2: EventContextHolder.get()})

        asyncio.get_event_loop().run_until_complete(
            asyncio.gather(*[event_0(), event_1(), event_2()])
        )
        for index, item in enumerate(_events):
            _items = list()
            self.assertEqual(index, list(item.keys())[0])
            _items.append(item)
        events = dict(ChainMap(*_events))
        self.assertNotEqual(events.get(0), events.get(1))
        self.assertNotEqual(events.get(0), events.get(2))
        self.assertNotEqual(events.get(1), events.get(2))



import json
import logging
import queue
import re
import sys
import typing
from logging.handlers import QueueHandler, QueueListener

from events_protocol.core.context import EventContextHolder
from datetime import datetime as dt


def _get_klass_name(klass: typing.Any) -> str:
    # Simple and effective
    return re.sub(r"|".join(map(re.escape, ["<class", "'", ">", " "])), "", str(klass))


_logger: logging.LoggerAdapter = None


def _get_logger() -> logging.LoggerAdapter:
    global _logger
    if _logger:
        return _logger
    logging.basicConfig(level=logging.INFO)
    _queue = queue.Queue(-1)
    _queue_handler = QueueHandler(_queue)
    _handler = logging.StreamHandler(sys.stdout)
    _queue_listener = QueueListener(_queue, _handler)

    _logger = logging.getLogger("gb.events_protocol")
    if _logger.hasHandlers():
        _logger.handlers.clear()
    _logger.addHandler(_queue_handler)
    _logger.propagate = False
    _queue_listener.start()
    return _logger


class JsonLogger(logging.LoggerAdapter):
    version: str = "NOTDEFINED"

    @classmethod
    def set_version(cls, version: str):
        cls.version = version

    def __init__(self, klass=None):
        self.logger = _get_logger()
        self.klass = _get_klass_name(klass)

    def log(self, level, msg, *args, **kwargs):
        if self.isEnabledFor(level):
            event_context = EventContextHolder.get()
            _msg = dict(
                severity=logging.getLevelName(level),
                logmessage=msg,
                EventID=event_context.id,
                FlowID=event_context.flow_id,
                UserId=event_context.user_id,
                Operation="{}:v{}".format(event_context.event_name, event_context.event_version),
                logger=self.klass,
                LoggerName=self.logger.name,
                logdate=dt.utcnow().isoformat(),
                ApplicationVersion=self.version,
            )

            extra = kwargs.pop("extra", None)
            if extra:
                if isinstance(extra, dict):
                    extra = json.dumps(extra)
                if not isinstance(extra, str):
                    extra_type = type(extra)
                    raise TypeError(
                        "Extra param needs to be dict or str, not {}".format(extra_type)
                    )
                _msg["extra"] = extra
            if level == logging.ERROR and kwargs.get("exc_info"):
                args = tuple()
                fmt = logging.Formatter()
                _exc = sys.exc_info()
                _msg["stackTrace"] = fmt.formatException(_exc)

                kwargs["exc_info"] = False
            msg = json.dumps(_msg)
            self.logger.log(level, msg, *args, **kwargs)



import requests

from typing import Dict

from requests.exceptions import Timeout, ReadTimeout, ConnectTimeout

from .exception.request_exception import TimeoutException, FailedDependencyException


class HttpClient:
    def post(self, url: str, headers: Dict[str, str], payload: str, timeout: int) -> str:
        try:
            response = requests.post(url=url, headers=headers, data=payload, timeout=timeout,)
        except (Timeout, ReadTimeout, ConnectTimeout) as exception:
            raise TimeoutException(f"Timeout calling {url}. Error {exception}")

        if not response.ok:
            raise FailedDependencyException(
                f"Failed dependency calling {url}. Error: {response.reason}"
            )

        return response.text



from abc import ABC, abstractmethod

from events_protocol.core.exception import MissingEventInformationException
from events_protocol.core.model.base import CamelPydanticMixin
from events_protocol.core.model.event import Event, RequestEvent, ResponseEvent
from events_protocol.core.model.base import ValidationError


class EventHandler(ABC):
    _SCHEMA: CamelPydanticMixin = None

    @abstractmethod
    def handle(cls, event: RequestEvent) -> ResponseEvent:
        raise NotImplementedError

    @classmethod
    def parse_event(cls, event: Event) -> CamelPydanticMixin:
        try:
            return event.payload_as(cls._SCHEMA)
        except ValidationError as exception:
            raise MissingEventInformationException(parameters=exception.to_dict())


class AsyncEventHandler(EventHandler, ABC):
    _SCHEMA: CamelPydanticMixin = None

    @abstractmethod
    async def handle(cls, event: RequestEvent) -> ResponseEvent:
        raise NotImplementedError



import json
from unittest import TestCase
from uuid import uuid4

from events_protocol.core.exception import EventParsingException
from events_protocol.core.logging.supressor import supress_log
from events_protocol.core.model.event import Event, CamelPydanticMixin, ResponseEvent
from events_protocol.core.model.event_type import EventErrorType
from tests.utils.sync import make_sync
from events_protocol.server.handler.event_handler import EventHandler
from events_protocol.server.handler.event_handler_registry import EventRegister
from events_protocol.server.parser.event_processor import AsyncEventProcessor, EventProcessor


class FakeSchema(CamelPydanticMixin):
    field: str


class FakeHandler(EventHandler):
    _SCHEMA = FakeSchema

    @classmethod
    async def handle(cls, event: Event):
        return event


class FakeRegister(EventRegister):
    event_version = 2
    event_name = "awesome:test:here"
    event_handler = FakeHandler


class TestAsyncEventProcessor(TestCase):
    @supress_log
    def test_parse_event_with_event_validation_error(self):
        test_json = json.dumps({"awesome": "test"})
        event_processor = AsyncEventProcessor()
        with self.assertRaises(EventParsingException):
            event_processor.parse_event(test_json)

    @supress_log
    def test_parse_event_with_event_parsing_error(self):
        test_json = r"{{wrong_json}"
        event_processor = AsyncEventProcessor()
        with self.assertRaises(EventParsingException):
            event_processor.parse_event(test_json)

    @supress_log
    def test_parse_event_with_valid_event(self):
        test_event = Event(name="event:test", version="2", id=str(uuid4()), flow_id=str(uuid4()))
        test_json = test_event.to_json()
        event_processor = AsyncEventProcessor()
        event = event_processor.parse_event(test_json)
        self.assertEqual(event, test_event)

    @supress_log
    @make_sync
    async def test_process_event_founding_none_event(self):
        test_event = Event(name="event:test", version="2", id=str(uuid4()), flow_id=str(uuid4()))
        test_json = test_event.to_json()
        event_processor = AsyncEventProcessor()
        event = await event_processor.process_event(test_json)
        response: ResponseEvent = ResponseEvent.from_json(event)
        self.assertTrue(response.is_error)
        self.assertEqual(response.event_type, EventErrorType.NOT_FOUND)

    @supress_log
    @make_sync
    async def test_process_event_founding_fake_event(self):
        FakeRegister.register_event()
        test_event = Event(
            name=FakeRegister.event_name,
            version=FakeRegister.event_version,
            id=str(uuid4()),
            flow_id=str(uuid4()),
            payload={},
        )
        test_json = test_event.to_json()
        event_processor = AsyncEventProcessor()
        response = await event_processor.process_event(test_json)
        self.assertEqual(test_json, response)

    @supress_log
    @make_sync
    def test_process_event_(self):
        test_event = Event(name="event:test", version="1", id=str(uuid4()), flow_id=str(uuid4()))
        test_json = test_event.to_json()
        event_processor = EventProcessor()
        event = event_processor.process_event(test_json)
        response: ResponseEvent = ResponseEvent.from_json(event)
        self.assertTrue(response.is_error)
        self.assertEqual(response.event_type, EventErrorType.NOT_FOUND)



from typing import Any, Dict, Optional

from .model.event_type import EventErrorType


class EventException(RuntimeError):
    _CODE: str = ""
    _TYPE: EventErrorType = EventErrorType.GENERIC

    def __init__(
        self, parameters: Dict[str, Optional[Any]], expected: bool = False,
    ):
        super().__init__(self._CODE, parameters, self._TYPE, expected)
        self.parameters = parameters
        self.expected = expected

    @property
    def code(self) -> str:
        return self._CODE

    @property
    def event_error_type(self) -> EventErrorType:
        return self._TYPE


class MessagebleEventException(EventException):
    def __init__(self, message: str):
        super().__init__(parameters={"message": message})


class EventNotFoundException(MessagebleEventException):
    _CODE = "EVENT_NOT_FOUND"
    _TYPE = EventErrorType.NOT_FOUND


class MissingEventInformationException(EventException):
    _CODE = "MISSING_FIELDS"
    _TYPE = EventErrorType.BAD_REQUEST


class EventParsingException(EventException):
    _CODE = "INVALID_COMMUNICATION_PROTOCOL"
    _TYPE = EventErrorType.BAD_PROTOCOL


class EventFailedDependencyException(MessagebleEventException):
    _CODE = "FAILED_DEPENDENCY"


class EventTimeoutException(MessagebleEventException):
    _CODE = "EVENT_TIMEOUT"



import typing
from abc import ABC

from .event_handler_discovery import EventDiscovery
from .event_handler import EventHandler


class EventRegister(ABC):
    event_name: str
    event_version: typing.Union[None, int]
    event_handler: EventHandler

    @classmethod
    def register_event(cls):
        """Call this method on startup application
        """
        EventDiscovery.add(cls.event_name, cls.event_handler, cls.event_version or 1)



from events_protocol.core.logging import JsonLogger


class LoggableMixin:
    logger = JsonLogger()

    def __new__(cls, *args, **kwargs):
        cls.logger = JsonLogger(cls)
        return super().__new__(cls)



import re
import typing

from events_protocol.core.exception import EventNotFoundException
from events_protocol.core.logging.mixins.loggable import LoggableMixin
from events_protocol.server.handler.event_handler import EventHandler, AsyncEventHandler


class EventDiscovery(LoggableMixin):
    _events: typing.Dict[
        typing.Tuple[str, int], typing.Union[EventHandler, AsyncEventHandler]
    ] = dict()
    _EVENT_NAME_STD: str = r"[a-z_]+[a-z]:[a-z_]+[a-z](:[a-z]+[a-z])*"

    @classmethod
    def add(cls, event_name: str, event_handler: EventHandler, version: int = 1) -> None:
        if not issubclass(event_handler, EventHandler):
            raise TypeError(f"event_handler must be of type EventHandler")
        res: re.Match = re.match(cls._EVENT_NAME_STD, event_name)
        if res is None or res.string != event_name:
            raise ValueError(
                f"Event name {event_name} does not meet the standard {cls._EVENT_NAME_STD}"
            )
        cls.logger.info(f"Registering event handler for {event_name} V{version}")
        cls._events[(event_name, version)] = event_handler

    @classmethod
    def get(cls, event_name: str, version: int = 1) -> EventHandler:
        event: typing.Union[EventHandler, None] = cls._events.get((event_name, version))
        if event is None:
            raise EventNotFoundException(
                f"Event Name {event_name} of version {version} was not found"
            )
        return event



import logging
import typing


def supress_log(f):
    def wrapper(*args, **kwargs):
        disable_logs()
        return f(*args, **kwargs)

    return wrapper


def disable_logs(log_names: typing.List[str] = []) -> None:
    filtered_logs = []
    log_keys = logging.Logger.manager.loggerDict.keys()
    if len(log_names) == 0:
        filtered_logs = log_keys
    else:
        for log in log_names:
            for key in log_keys:
                if key.startswith(log):
                    filtered_logs.append(key)
    for filter_key in filtered_logs:
        log = logging.getLogger(filter_key)
        log.disabled = True
        log.addHandler(logging.NullHandler())
        log.propagate = False
        log.addFilter(lambda record: False)



from os.path import dirname, join
from setuptools import setup, find_packages

import events_protocol


with open(join(dirname(__file__), "README.md")) as _file:
    long_description = _file.read()


setup(
    name="events-protocol",
    version=events_protocol.__version__,
    author="Guiabolso",
    description="Library to be a Client and Server using event protocol",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/GuiaBolso/events-protocol-python",
    license="Apache-2.0",
    packages=find_packages(),
    install_requires=[
        "certifi>=2019.11.28",
        "chardet>=3.0.4",
        "idna>=2.8",
        "pydantic>=1.4",
        "requests>=2.22.0",
        "urllib3>=1.25.8",
    ],
    extras_require={"doc": ["Sphinx==2.4.1", "sphinx-autoapi==1.2.1",],},
    project_urls={
        "Documentation": "https://events-protocol.readthedocs.io/en/stable/",
        "Source Code": "https://github.com/GuiaBolso/events-protocol-python",
    },
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: Unix",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.7",
)



from typing import Any, Dict, Generic, Optional
from uuid import uuid4

from .base import CamelPydanticMixin
from .event_type import EventErrorType, EventSuccessType, EventType

PayloadType = Dict[str, Any]


class Event(CamelPydanticMixin):
    name: str
    version: int
    payload: PayloadType = dict()
    id: Optional[str] = str(uuid4())
    flow_id: Optional[str] = str(uuid4())
    identity: Optional[Dict[str, Any]] = dict()
    auth: Optional[Dict[str, Any]] = dict()
    metadata: Optional[Dict[str, Any]] = dict()

    def payload_as(self, clazz: CamelPydanticMixin) -> CamelPydanticMixin:
        return clazz(**self.payload)

    def identity_as(self, clazz: Generic) -> Generic:
        return clazz.from_dict(self.identity)

    def auth_as(self, clazz: Generic) -> Generic:
        return clazz.from_dict(self.auth)

    @property
    def user_id(self) -> Optional[int]:
        if self.identity is not None:
            user = self.identity.get("userId")
            if user:
                return int(user)

    @property
    def origin(self) -> Optional[str]:
        if self.metadata is not None:
            return self.metadata.get("origin")


class ResponseEvent(Event):
    @staticmethod
    def from_event(
        event: Event, event_type: EventType = EventSuccessType.SUCCESS
    ) -> "ResponseEvent":
        response_event = ResponseEvent.from_object(event)
        response_event.name = f"{response_event.name}:{event_type}"
        return response_event

    @property
    def is_success(self) -> bool:
        return self.event_type == EventSuccessType.SUCCESS

    @property
    def is_redirect(self) -> bool:
        return self.event_type == EventSuccessType.REDIRECT

    @property
    def is_error(self) -> bool:
        return EventErrorType.is_in(self.event_type)

    @property
    def _event_name(self) -> str:
        return self.name.split(":")[-1]

    @property
    def event_type(self) -> EventType:
        return EventSuccessType.get_type(self._event_name) or EventErrorType.get_type(
            self._event_name
        )

    @property
    def error_type(self) -> EventErrorType:
        if self.is_error:
            return EventErrorType.get_type(self._event_name)
        raise ValueError("This is not an error event.")


class RequestEvent(Event):
    pass


class EventMessage(CamelPydanticMixin):
    code: str
    parameters: Dict[str, Optional[Any]]



class BaseRequestException(Exception):
    def __init__(self, message: str):
        super().__init__(message)


class TimeoutException(BaseRequestException):
    pass


class FailedDependencyException(BaseRequestException):
    pass


class BadProtocolException(BaseRequestException):
    pass



from unittest import TestCase
from unittest.mock import patch, MagicMock

from requests.exceptions import Timeout, ReadTimeout, ConnectTimeout

from events_protocol.client.http import HttpClient
from events_protocol.client.exception.request_exception import (
    TimeoutException,
    FailedDependencyException,
)


class HttpClientTest(TestCase):
    def setUp(self):
        self.http_client = HttpClient()

    @patch("events_protocol.client.http.requests")
    def test_success_response(self, requests):
        response_mock = MagicMock()
        response_mock.status_code = 200
        response_mock.text = "THIS IS A TEST"

        requests.post.return_value = response_mock

        returned_value = self.http_client.post(
            url="http://test.com/", headers=None, payload=None, timeout=1000
        )

        self.assertEqual(response_mock.text, returned_value)

    @patch("events_protocol.client.http.requests")
    def test_timeout_exception(self, requests):
        requests.post.side_effect = Timeout()

        with self.assertRaises(TimeoutException):
            self.http_client.post(url="http://test.com/", headers=None, payload=None, timeout=1000)

    @patch("events_protocol.client.http.requests")
    def test_read_timeout_exception(self, requests):
        requests.post.side_effect = ReadTimeout()

        with self.assertRaises(TimeoutException):
            self.http_client.post(url="http://test.com/", headers=None, payload=None, timeout=1000)

    @patch("events_protocol.client.http.requests")
    def test_connect_timeout_exception(self, requests):
        requests.post.side_effect = ConnectTimeout()

        with self.assertRaises(TimeoutException):
            self.http_client.post(url="http://test.com/", headers=None, payload=None, timeout=1000)

    @patch("events_protocol.client.http.requests")
    def test_not_ok_response(self, requests):
        response_mock = MagicMock()
        response_mock.ok = False

        requests.post.return_value = response_mock

        with self.assertRaises(FailedDependencyException):
            self.http_client.post(
                url="http://test.com/", headers=None, payload=None, timeout=1000,
            )



import typing
from enum import Enum, unique


class EventType(Enum):
    @classmethod
    def is_in(cls, item: typing.Any) -> bool:
        try:
            cls(item)
            return True
        except ValueError:
            return False

    def __str__(self) -> str:
        return str(self.value)


@unique
class EventErrorType(EventType):
    GENERIC = "error"
    BAD_PROTOCOL = "badProtocol"
    BAD_REQUEST = "badRequest"
    UNAUTHORIZED = "unauthorized"
    NOT_FOUND = "notFound"
    FORBIDDEN = "forbidden"
    USER_DENIED = "userDenied"
    RESOURCE_DENIED = "resourceDenied"
    EXPIRED = "expired"
    UNKNOWN = "unknown"

    @classmethod
    def get_type(cls, error: str) -> "EventErrorType":
        try:
            return cls(error)
        except ValueError:
            return cls.UNKNOWN


@unique
class EventSuccessType(EventType):
    SUCCESS = "response"
    REDIRECT = "redirect"

    @classmethod
    def get_type(cls, success: str) -> typing.Optional["EventSuccessType"]:
        try:
            return cls(success)
        except ValueError:
            return None



import json

from events_protocol.core.context import EventContextHolder
from events_protocol.core.exception import (
    EventException,
    EventParsingException,
)
from events_protocol.core.logging.mixins.loggable import LoggableMixin
from events_protocol.core.model.base import ValidationError
from events_protocol.core.model.event import Event, ResponseEvent
from events_protocol.server.handler.event_handler import EventHandler, AsyncEventHandler
from events_protocol.server.handler.event_handler_discovery import EventDiscovery

from events_protocol.core.builder import EventBuilder


class EventProcessor(LoggableMixin):
    event_discovery = EventDiscovery
    event_validator = Event

    @classmethod
    def process_event(cls, raw_event: str) -> str:
        event = None
        try:
            event: Event = cls.parse_event(raw_event)
            with EventContextHolder.with_context(
                event.id, event.flow_id, event.name, event.version
            ) as _:
                event_handler: EventHandler = EventDiscovery.get(event.name, event.version)
                response: ResponseEvent = event_handler.handle(event)
                return response.to_json()
        except EventException as exception:
            return EventBuilder.error_for(exception, event).to_json()

    @classmethod
    def parse_event(cls, str_event: str) -> Event:
        try:
            event = cls.event_validator.from_json(str_event)
            return event

        except ValidationError as exception:
            cls.logger.exception(
                "Error in event schema validation",
                exception,
                extra=dict(event=json.dumps(str_event)),
            )
            raise EventParsingException(exception.to_dict())
        except Exception as exception:
            cls.logger.exception(
                "Error in parser schema procedure",
                exception,
                extra=dict(event=json.dumps(str_event)),
            )
            raise EventParsingException(dict(error="Unknown error"))


class AsyncEventProcessor(EventProcessor):
    @classmethod
    async def process_event(cls, raw_event: str) -> str:
        event = None
        try:
            event: Event = cls.parse_event(raw_event)
            async with EventContextHolder.with_async_context(
                event.id, event.flow_id, event.name, event.version, event.user_id
            ) as _:
                event_handler: AsyncEventHandler = EventDiscovery.get(event.name, event.version)
                response: ResponseEvent = await event_handler.handle(event)
                return response.to_json()
        except EventException as exception:
            return EventBuilder.error_for(exception, event).to_json()



import typing
from uuid import uuid4

from events_protocol.core.exception import EventException
from events_protocol.core.model.event import (
    Event,
    EventMessage,
    PayloadType,
    ResponseEvent,
)
from events_protocol.core.model.event_type import EventSuccessType
from events_protocol.core.logging.mixins.loggable import LoggableMixin


class EventBuilder(LoggableMixin):
    @classmethod
    def error_for(
        cls,
        exception: typing.Union[EventException],
        event: typing.Optional[Event] = Event(name="", version=1, id=str(uuid4())),
        id_flow=str(uuid4()),
        loggable=True,
    ) -> ResponseEvent:
        event_name = (
            f"{event.name}:{exception.event_error_type}"
            if event.name
            else f"{exception.event_error_type}"
        )

        event_message = EventMessage(code=exception.code, parameters=exception.parameters).to_dict()
        response_event = ResponseEvent.from_object(event)
        response_event.name = event_name
        response_event.payload = event_message
        if loggable:
            cls.logger.error("Event finished with error", extra=response_event.json())
        return response_event

    @classmethod
    def response_for(
        cls,
        event: Event,
        payload: PayloadType,
        event_type: EventSuccessType = EventSuccessType.SUCCESS,
        loggable=True,
    ) -> ResponseEvent:
        response_event = ResponseEvent.from_object(event)
        response_event.name = f"{response_event.name}:{event_type}"
        response_event.payload = payload
        return response_event
